//= require <external/IconCombo>

Ext.namespace("dSS", "dSS.config", "dSS.devprops");

// form *must* react to the submit event and fire a formsubmitted
// event in response; it *must* also fire a formloaded event once all d
// has been received
//
// parameters: dsid, group (configured joker group)
dSS.devprops.MultiButtonConfigForm = Ext.extend(Ext.form.FormPanel, {
    constructor: function(config) {
        if (config.dsid === undefined) {
            Ext.MessageBox.alert(dSS._("Error"), ("Missing dSID"));
            return;
        }

        this.dsid = config.dsid;
        this.buttonConfig = config.buttonConfig;
        this.buttonIndex = config.buttonIndex;

        this.addEvents('formsubmitted', 'formloaded');

        this.buttonInputTypeSelection = new Ext.form.ComboBox({
            typeAhead: false,
            triggerAction: 'all',
            lazyRender:true,
            forceSelection: true,
            editable: false,
            width: 130,
            mode: 'local',
            fieldLabel: dSS._("Button function"),
            store: new Ext.data.ArrayStore({
                id: 0,
                fields:
                [
                    'id',
                    'displayText'
                ],
                data:
                [
                    [dSS.config.ButtonInputMode.BUTTONINPUT_1WAY, dSS._("1-way")],
                    [dSS.config.ButtonInputMode.BUTTONINPUT_2WAY_DOWN, dSS._("2-way down")],
                    [dSS.config.ButtonInputMode.BUTTONINPUT_2WAY_UP, dSS._("2-way up")]

                ]
            }),
            valueField: 'id',
            displayField: 'displayText'
        });

        Ext.apply(this, {
            bubbleEvents: [ 'formsubmitted', 'formloaded' ],
            padding: 5,
            scope: this,
            items:
            [
                {
                    xtype: 'panel',
                    border: false,
                    height: 22,
                    width: 355,
                    layout: 'form',
                    items:
                    [
                        this.buttonInputTypeSelection
                    ]
                }
            ]
        });

        this.firstLoad = true;

        dSS.devprops.MultiButtonConfigForm.superclass.constructor.apply(this, arguments);

        this.on('afterrender', function() {
                this.fireEvent('formloaded', true);
            },
            this);
        
        this.translateButtonconfigToSelection = function(buttonConfig) {
            if ((buttonConfig >= dSS.config.deviceParams.BUTTONINPUT_2WAY_DW_WITH_INPUT1) &&
                (buttonConfig <= dSS.config.deviceParams.BUTTONINPUT_2WAY_DW_WITH_INPUT4)) {
                 return dSS.config.ButtonInputMode.BUTTONINPUT_2WAY_DOWN;
            } else if ((buttonConfig >= dSS.config.deviceParams.BUTTONINPUT_2WAY_UP_WITH_INPUT1) &&
                       (buttonConfig <= dSS.config.deviceParams.BUTTONINPUT_2WAY_UP_WITH_INPUT4)) {
                return dSS.config.ButtonInputMode.BUTTONINPUT_2WAY_UP;
            } else {
                return dSS.config.ButtonInputMode.BUTTONINPUT_1WAY;
            }
        };

        this.buttonInputTypeSelection.on('afterrender', function() {
                if (this.closed !== true) {
                    this.buttonInputTypeSelection.setValue(this.translateButtonconfigToSelection(this.buttonConfig));
                }
            },
            this);

        this.on('beforedestroy', function() {
                this.closed = true;
            },
            this);

        this.on('submit', function() {
                var value = this.buttonInputTypeSelection.getValue();
                var dsid = this.dsid;
                var buttonConfig = this.buttonConfig;
                if (value == this.translateButtonconfigToSelection(buttonConfig)) {
                    this.fireEvent('formsubmitted');
                    return;
                }
                
                var buttonInputMode = 0;
                var buttonInputModePartner = 0;

                if (value === dSS.config.ButtonInputMode.BUTTONINPUT_2WAY_DOWN) {
                    if (this.buttonIndex == 0) {
                        buttonInputMode = dSS.config.deviceParams.BUTTONINPUT_2WAY_DW_WITH_INPUT2;
                        buttonInputModePartner = dSS.config.deviceParams.BUTTONINPUT_2WAY_UP_WITH_INPUT1;
                    } else {
                        buttonInputMode = dSS.config.deviceParams.BUTTONINPUT_2WAY_DW_WITH_INPUT4;
                        buttonInputModePartner = dSS.config.deviceParams.BUTTONINPUT_2WAY_UP_WITH_INPUT3;
                    }
                } else if (value === dSS.config.ButtonInputMode.BUTTONINPUT_2WAY_UP) {
                    if (this.buttonIndex == 0) {
                        buttonInputMode = dSS.config.deviceParams.BUTTONINPUT_2WAY_UP_WITH_INPUT2;
                        buttonInputModePartner = dSS.config.deviceParams.BUTTONINPUT_2WAY_DW_WITH_INPUT1;
                    } else {
                        buttonInputMode = dSS.config.deviceParams.BUTTONINPUT_2WAY_UP_WITH_INPUT4;
                        buttonInputModePartner = dSS.config.deviceParams.BUTTONINPUT_2WAY_DW_WITH_INPUT3;
                    }
                } else {
                    buttonInputMode = dSS.config.deviceParams.BUTTONINPUT_STANDARD;
                    buttonInputModePartner = dSS.config.deviceParams.BUTTONINPUT_STANDARD;
                }
                
                var deferDelay = 2000;

                dSS.util.genericRequest.defer(deferDelay,
                    this,
                    [
                         {
                            url: '/json/device/setButtonInputMode',
                            params:
                            {
                                dsid : dsid,
                                modeID : buttonInputMode
                            },
                            failureMsg:
                                dSS._("Could not set button input mode of device"),
                            scope: this,
                            evt: 'formsubmitted'
                        },
                        function(success) {
                            if (success === true) {
                                var devStore = dSS.data.Loader.getDeviceStore();
                                var record = devStore.getById(dsid);
                                if (record !== undefined) {
                                    record.set('buttonInputMode', buttonInputMode);
                                    record.commit();
                                }
                                dSS.MessageBus.fireEvent('scenesmarkdirty');
                            }
                        }
                    ]
                );
                
                var devStore = dSS.data.Loader.getDeviceStore();
                var self = devStore.getById(dsid);
                var partner = devStore.getById(dSS.util.getDSIDPlusOne(dsid));

                deferDelay += 500;
                dSS.util.genericRequest.defer(deferDelay,
                    this,
                    [
                         {
                            url: '/json/device/setButtonInputMode',
                            params:
                            {
                                dsid : partner.get('dsid'),
                                modeID : buttonInputModePartner
                            },
                            failureMsg:
                                dSS._("Could not set button input mode of device"),
                            scope: this,
                            evt: 'formsubmitted'
                        },
                        function(success) {
                            if (success === true) {
                                var devStore = dSS.data.Loader.getDeviceStore();
                                var record = devStore.getById(dSS.util.getDSIDPlusOne(dsid));
                                if (record !== undefined) {
                                    record.set('buttonInputMode', buttonInputModePartner);
                                    record.commit();
                                }
                                dSS.MessageBus.fireEvent('scenesmarkdirty');

                                var dSMStore = dSS.data.Loader.getModulatorStore();
                                var i = dSMStore.find('dsid', record.get('meterDSID'));
                                if (i > -1) {
                                    var dSM = dSMStore.getAt(i);
                                    dSM.set('numDevices', dSM.get('numDevices') + (dSS.util.is2WaySlave(buttonInputModePartner) ? -1 : 1));
                                    dSM.commit();
                                    dSS.MessageBus.fireEvent('deviceattachementchanged', dSM);
                                }
                            }
                        }
                    ]
                );
                
                if (value === dSS.config.ButtonInputMode.BUTTONINPUT_1WAY) {
                    return;
                }
                
                deferDelay += 500;
                var changeZone = function(scope) {
                    if (self.get('zoneId') !== partner.get('zoneId')) {
                        dSS.util.genericRequest(
                             {
                                url: '/json/structure/zoneAddDevice',
                                params:
                                {
                                    deviceID : partner.get('dsid'),
                                    zone : self.get('zoneId')
                                },
                                failureMsg:
                                    dSS._("Could not move slave device into same zone"),
                                scope: scope,
                                evt: 'formsubmitted'
                            },
                            function(success) {
                                if (success === true) {
                                    var devStore = dSS.data.Loader.getDeviceStore();
                                    var record = devStore.getById(dSS.util.getDSIDPlusOne(dsid));
                                    var self = devStore.getById(dsid);
                                    if (record !== undefined) {
                                        record.set('zoneId', self.get('zoneId'));
                                        record.commit();
                                    }
                                    dSS.MessageBus.fireEvent('scenesmarkdirty');
                                }
                            }
                        );
                    }
                };
                changeZone.defer(deferDelay, this, [this]);
                
                deferDelay += 500;
                var changeButtonId = function(scope) {
                    if (self.get('buttonID') !== partner.get('buttonID')) {
                        dSS.util.genericRequest(
                            {
                                url: '/json/device/setButtonID',
                                params:
                                {
                                    dsid : partner.get('dsid'),
                                    buttonID : self.get('buttonID')
                                },
                                failureMsg:
                                        dSS._("Could not save pushbutton settings of slave device"),
                                scope: scope,
                                evt: 'formsubmitted'
                            },
                            function(success) {
                                if (success === true) {
                                    var devStore = dSS.data.Loader.getDeviceStore();
                                    var record = devStore.getById(dSS.util.getDSIDPlusOne(dsid));
                                    var self = devStore.getById(dsid);
                                    if (record !== undefined) {
                                        record.set('buttonID', self.get('buttonID'));
                                        record.commit();
                                    }
                                    dSS.MessageBus.fireEvent('scenesmarkdirty');
                                }
                            }
                        );
                    }
                };
                changeButtonId.defer(deferDelay, this, [this]);
                
                deferDelay += 500;
                var changeJockerGroup = function(scope) {
                    if (dSS.util.getJokerGroup(self.get('groups')) !== dSS.util.getJokerGroup(partner.get('groups'))) {
                        dSS.util.genericRequest(
                            {
                                url: '/json/device/setJokerGroup',
                                params:
                                {
                                    dsid : partner.get('dsid'),
                                    groupID : dSS.util.getJokerGroup(self.get('groups'))
                                },
                                failureMsg:
                                        dSS._("Could not save group settings of slave device"),
                                scope: scope,
                                evt: 'formsubmitted'
                            },
                            function(success) {
                                if (success === true) {
                                    var devStore = dSS.data.Loader.getDeviceStore();
                                    var record = devStore.getById(dSS.util.getDSIDPlusOne(dsid));
                                    var self = devStore.getById(dsid);
                                    if (record !== undefined) {
                                        record.set('groups', self.get('groups'));
                                        record.commit();
                                    }
                                    dSS.MessageBus.fireEvent('scenesmarkdirty');
                                }
                            }
                        );
                    }
                };
                changeJockerGroup.defer(deferDelay, this, [this]);
        },
        this);
    }
});

Ext.reg('dssdevmultibuttonconfigform', dSS.devprops.MultiButtonConfigForm);

