Ext.namespace("dSS", "dSS.config", "dSS.devprops");

dSS.devprops.CheckShowSceneProperties = function(matrix) {
    return (matrix.blink || matrix.dontcare);
};

// form *must* react to the submit event and fire a formsubmitted
// event in response; it *must* also fire a formloaded event once all d
// has been received
//
// parameters: dsid
dSS.devprops.ScenePropertiesForm = Ext.extend(Ext.form.FormPanel, {
    constructor: function(config) {
        if (config.dsid === undefined) {
            Ext.Msg.alert(dSS._("Error"), ("Missing dSID"));
            return;
        }

        if (config.scene === undefined) {
            Ext.Msg.alert(dSS._("Error"), ("Missing scene information"));
            return;
        }

        this.dsid = config.dsid;
        this.scene = config.scene;

        this.addEvents('formsubmitted', 'formloaded');

        var checkboxes = [];

        if (config.visibilityMatrix === undefined) {
            Ext.Msg.alert(("Error"), ("Missing config options"));
            return;
        }

        if (config.visibilityMatrix.blink === true) {
            checkboxes.push({
                boxLabel: dSS._("Blink when calling scene"),
                name: 'cb_sceneprops_' + config.dsid,
                id: 'blink'
            });
        }

        if (config.visibilityMatrix.dontcare === true) {
            checkboxes.push({
                boxLabel: dSS._("Retain output"),
                name: 'cb_sceneprops_' + config.dsid,
                id: 'dontcare'
            });
        }

        this.standardSettings = new Ext.form.CheckboxGroup({
            columns: 1,
            items: checkboxes
        });

        // update output mode string once we know the correct value
        this.infoPanel = dSS.util.CreateKeyValPanel([
            {
                key: dSS._("Device name"),
                value: dSS.util.getName(config.deviceName)
            },
            {
                key: dSS._("Device ID"),
                value: dSS.util.shortDSID(config.dsid)
            },
           { key: dSS._("Output mode"), value: '...' }
        ]);
        var items =
        [
            this.infoPanel,
             {
                xtype: 'panel',
                layout: 'form',
                ref: 'standard',
                id: 'standard',
                items:
                [
                    {
                        xtype: 'box',
                        height: 5
                    },
                    this.standardSettings
                ]
            }
        ];

        // advanced properties available
        if ((config.visibilityMatrix.ledauto === true) ||
            (config.visibilityMatrix.leddark === true) ||
            (config.visibilityMatrix.transt === true)) {
            // spacer panel
            items.push({
                xtype: 'panel',
                border: false,
                height: 10,
                bodyCfg: {
                    cls: 'x-window-mc'
                }
            });

            var acbitems = [];

            var leditems = [];

            if (config.visibilityMatrix.ledauto === true) {
                leditems.push(
                    {
                        boxLabel: dSS._("auto"),
                        name: 'rb_led_sceneprops_' + config.dsid,
                        id: 'led' + dSS.config.deviceParams.LED_CONFIG_AUTO_DARK,
                        inputValue: dSS.config.deviceParams.LED_CONFIG_AUTO_DARK
                    });
            }

            if (config.visibilityMatrix.leddark === true) {
                leditems.push(
                    {
                        boxLabel: dSS._("on"),
                        name: 'rb_led_sceneprops_' + config.dsid,
                        id: 'led' + dSS.config.deviceParams.LED_CONFIG_ON,
                        inputValue: dSS.config.deviceParams.LED_CONFIG_ON
                    });
                leditems.push(
                    {
                        boxLabel: dSS._("dark"),
                        name: 'rb_led_sceneprops_' + config.dsid,
                        id: 'led' + dSS.config.deviceParams.LED_CONFIG_AUTO_DARK,
                        inputValue: dSS.config.deviceParams.LED_CONFIG_AUTO_DARK
                    });
            }

            if ((config.visibilityMatrix.ledauto === true) ||
                (config.visibilityMatrix.leddark === true)) {

                leditems.push(
                    {
                        boxLabel: dSS._("off"),
                        name: 'rb_led_sceneprops_' + config.dsid,
                        id: 'led' + dSS.config.deviceParams.LED_CONFIG_OFF,
                        inputValue: dSS.config.deviceParams.LED_CONFIG_OFF
                    });

                this.ledSettings = new Ext.form.RadioGroup({
                    columns: 1,
                    padding: 5,
                    fieldLabel: dSS._("LED mode"),
                    items: leditems
                });

                acbitems.push(this.ledSettings);
            }

            if (config.visibilityMatrix.transt === true) {
                this.dimSettings = new Ext.form.RadioGroup({
                    columns: 1,
                    padding: 5,
                    fieldLabel: dSS._("Transition time"),
                    items:
                    [
                        {
                            boxLabel: dSS._("standard"),
                            name: 'rb_dim_sceneprops_' + config.dsid,
                            id: 'dim' +
                             dSS.config.deviceParams.DIM_CONFIG_SELECT_STANDARD,
                            inputValue:
                             dSS.config.deviceParams.DIM_CONFIG_SELECT_STANDARD
                        },
                        {
                            boxLabel: dSS._("slow"),
                            name: 'rb_dim_sceneprops_' + config.dsid,
                            id: 'dim' +
                                dSS.config.deviceParams.DIM_CONFIG_SELECT_SLOW,
                            inputValue:
                                dSS.config.deviceParams.DIM_CONFIG_SELECT_SLOW
                        },
                        {
                            boxLabel: dSS._("custom"),
                            name: 'rb_dim_sceneprops_' + config.dsid,
                            id: 'dim' +
                               dSS.config.deviceParams.DIM_CONFIG_SELECT_CUSTOM,
                            inputValue:
                               dSS.config.deviceParams.DIM_CONFIG_SELECT_CUSTOM
                        }
                    ]
                });

                acbitems.push(this.dimSettings);
            }

            items.push({
                xtype: 'panel',
                title: dSS._("Show advanced properties"),
                collapsed: true,
                collapsible: true,
                collapseFirst: true,
                animCollapse: false,
                floatable: false,
                titleCollapse: true,
                ref: 'advancedPanel',
                disabled: true,
                layout: 'form',
                padding: 5,
                listeners:
                {
                    collapse: function(p) {
                        p.setTitle(
                              dSS._("Show advanced properties"));
                    },
                    expand: function(p) {
                        p.setTitle(
                              dSS._("Hide advanced properties"));
                    }
                },
                items: [ acbitems ]
            });
        }

        this.canSubmit = false;

        Ext.apply(this, {
            bubbleEvents: [ 'formsubmitted', 'formloaded' ],
            scope: this,
            disabled: true,
            border: false,
            bodyCfg: {
                cls: 'x-window-mc'
             },

            params:
            {
                'class' : 0x80,
                index : 0x80 + config.scene,
                dsid : config.dsid
            },
            items: items
        });

        dSS.devprops.ScenePropertiesForm.superclass.constructor.apply(this,
                                                                     arguments);

        this.on('beforedestroy', function() {
                if (this.AjaxRequest !== undefined) {
                    Ext.Ajax.abort(this.AjaxRequest);
                }
                this.closed = true;
            },
            this);

        this.on('submit', function() {
            if (this.canSubmit !== true) {
                this.fireEvent('formsubmitted');
                return;
            }
            var value = {};

            if (this.standardSettings.items.get('blink').getValue() === true) {
                value.flashMode = 1;
            } else {
                value.flashMode = 0;
            }

            if (this.standardSettings.disabled === false) {
                if (this.standardSettings.items.get('dontcare').getValue()
                                                                    === true) {
                    value.dontCare = 1;
                } else {
                    value.dontCare = 0;
                }
            }

            if (this.ledSettings !== undefined) {
                var ledval = this.ledSettings.getValue().getRawValue();

                if ((ledval ===
                    dSS.config.deviceParams.LED_CONFIG_AUTO_DARK.toString()) ||
                    (ledval ===
                        dSS.config.deviceParams.LED_CONFIG_OFF.toString()) ||
                    (ledval ===
                        dSS.config.deviceParams.LED_CONFIG_ON.toString())) {
                    value.ledconIndex = ledval;
                }
            }

            if (this.dimSettings !== undefined) {
                var dimval = this.dimSettings.getValue().getRawValue();
                if ((dimval ===
                    dSS.config.deviceParams.DIM_CONFIG_SELECT_STANDARD.toString()) ||
                    (dimval ===
                        dSS.config.deviceParams.DIM_CONFIG_SELECT_SLOW.toString()) ||
                    (dimval ===
                        dSS.config.deviceParams.DIM_CONFIG_SELECT_CUSTOM.toString())) {
                    value.dimtimeIndex = dimval;
                }
            }

            if ((value.dontCare === this.loadedValueObj.dontCare) &&
                (value.flashMode === this.loadedValueObj.flashMode) &&
                (value.ledconIndex === this.loadedValueObj.ledconIndex) &&
                (value.dimtimeIndex === this.loadedValueObj.dimtimeIndex)) {
                this.fireEvent('formsubmitted');
                return;
            }

            value.dsid = this.dsid;
            value.sceneID = this.scene;

            this.AjaxRequest = dSS.util.genericRequest({
                url: '/json/device/setSceneMode',
                params: value,
                failureMsg: dSS._("Could not save device scene settings"),
                maskEl: this.el,
                maskMsg: dSS._("Saving device scene settings"),
                scope: this,
                evt: 'formsubmitted'
            });

        },
        this);

        // if masking is done in afterrender, the animation is not positioned
        // correctly; however we can not do the ajax call in afterlayout,
        // because it seems that afterlayout gets called twice
        this.on('afterlayout', function() {
                this.el.mask(dSS._("Loading device scene settings") + "...",
                             'x-mask-loading');
            },
            this);

        this.on('afterrender', function() {
                this.loadFirst();
            },
            this);


    },

    loadFirst: function() {
        this.AjaxRequest = Ext.Ajax.request({
            url: '/json/device/getConfig',
            method: 'GET',
            params: {
                'class': 3,
                index: 0,
                dsid: this.dsid
            },
            scope: this,
            success: function(response) {
                this.AjaxRequest = undefined;
                if (this.closed === true) {
                    return;
                }

                try {
                    var jsonData = Ext.util.JSON.decode(response.responseText);
                    if (jsonData.ok !== true) {
                        Ext.Msg.alert(dSS._("Error"),
                                      dSS._("Settings disabled") +
                                      ': ' + jsonData.message);
                        this.el.unmask();
                        this.fireEvent('formloaded', this.canSubmit);
                        return;
                    }

                    if (this.closed === true) {
                        return;
                    }
                    this.loadSecond(jsonData.result.value);
                } catch (error) {
                    if (this.closed === true) {
                        return;
                    }

                    Ext.Msg.alert(dSS._("Error"),
                            dSS._("Could not load output mode settings") +
                            ': ' + error);
                    this.el.unmask();
                    this.fireEvent('formloaded', this.canSubmit);
                    return;
                }
            },
            failure: function(response) {
                this.AjaxRequest = undefined;
                if (this.closed === true) {
                    return;
                }
                Ext.Msg.alert(dSS._("Error"),
                              dSS._("Could not load output mode settings"));
                this.el.unmask();
                this.fireEvent('formloaded', this.canSubmit);
            }
        });

    },
    loadSecond: function(mode) {
        if (this.closed === true) {
            return;
        }

        this.infoPanel.items[this.infoPanel.items.length-1].setText(dSS._(dSS.config.OutModeStrings[mode]));
        if (mode == dSS.config.deviceParams.OUTPUT_MODE_DISABLED) {
            var msg = dSS._("Configuration not possible: output" +
                    " mode of the device is disabled");

            if (this.outModeTip !== undefined) {
                this.outModeTip.destroy();
            }

            // window may got closed inbetween
            if (this.standardSettings !== undefined) {
                this.outModeTip = new Ext.ToolTip({
                    html: msg,
                    draggable: false,
                    title: dSS._("Notice"),
                    anchor: 'left',
                    target: 'standard',
                    dismissDelay: 10000
                });

                this.outModeTip.show();
            }
        }

        var errMsg = dSS._("Configuration not possible: could not load " +
                    "device scene settings");

        this.AjaxRequest = Ext.Ajax.request(
            {
                url: '/json/device/getSceneMode',
                method: 'GET',
                params:
                {
                    sceneID : this.scene,
                    dsid : this.dsid
                },
                scope: this,
                success: function(response) {
                    this.AjaxRequest = undefined;
                    if (this.closed === true) {
                        return;
                    }

                    try {
                        var jsonData = Ext.util.JSON.decode(
                                            response.responseText);
                        if (jsonData.ok !== true) {
                            Ext.Msg.alert(dSS._("Error"), errMsg);
                            this.el.unmask();
                            this.fireEvent('formloaded', this.canSubmit);
                            return;
                        }

                        var val = jsonData.result;

                        this.loadedValueObj = val;
                        this.standardSettings.setValue([val.flashMode,
                                                        val.dontCare]);

                        if (this.ledSettings !== undefined) {
                            this.ledSettings.setValue('led' + val.ledconIndex, true);
                        }

                        if (this.dimSettings !== undefined) {
                            this.dimSettings.setValue('dim' + val.dimtimeIndex, true);
                        }

                        this.el.unmask();
                        if (mode ==
                                dSS.config.deviceParams.OUTPUT_MODE_DISABLED) {
                            this.ledSettings.enable();
                        } else {
                            this.enable();
                        }
                        if (this.advancedPanel !== undefined) {
                            this.advancedPanel.enable();
                        }
                        this.canSubmit = true;
                        this.fireEvent('formloaded', this.canSubmit);

                    } catch (error) {
                        if (this.closed === true) {
                            return;
                        }
                        Ext.Msg.alert(dSS._("Error"),
                                      dSS._("Configuration not possible") +
                                      ': ' + error);
                        this.el.unmask();
                        this.fireEvent('formloaded', this.canSubmit);
                    }
                },
                failure: function(result, request) {
                    this.AjaxRequest = undefined;
                    if (this.closed === true) {
                        return;
                    }
                    Ext.Msg.alert(dSS._("Error"), errMsg);
                    this.el.unmask();
                    this.fireEvent('formloaded', this.canSubmit);
                }
        });
    }
});

Ext.reg('dssdevscenepropsform', dSS.devprops.ScenePropertiesForm);

